@@ -81,37 +81,6 @@ impl<T: EventListener> Context<T> {
         self.renderable_content.hyperlink_range.is_some()
     }
 
-    #[inline]
-    pub fn renderable_content(&mut self) -> &RenderableContent {
-        let mut has_ime = false;
-        if let Some(preedit) = self.ime.preedit() {
-            if let Some(content) = preedit.text.chars().next() {
-                self.renderable_content.cursor.content = content;
-                self.renderable_content.cursor.is_ime_enabled = true;
-                has_ime = true;
-            }
-        }
-
-        if !has_ime {
-            self.renderable_content.cursor.is_ime_enabled = false;
-            self.renderable_content.cursor.content =
-                self.renderable_content.cursor.content_ref;
-        }
-
-        let terminal = self.terminal.lock();
-        self.renderable_content.update(
-            terminal.visible_rows(),
-            terminal.display_offset(),
-            terminal.cursor(),
-            terminal.is_fully_damaged(),
-            terminal.colors,
-            terminal.blinking_cursor,
-        );
-        drop(terminal);
-
-        &self.renderable_content
-    }
-
     #[inline]
     pub fn cursor_from_ref(&self) -> Cursor {
         Cursor {
@@ -659,8 +628,8 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     }
 
     #[inline]
-    pub fn grid_objects(&self) -> Vec<Object> {
-        self.contexts[self.current_index].objects()
+    pub fn extend_with_grid_objects(&self, target: &mut Vec<Object>) {
+        self.contexts[self.current_index].extend_with_objects(target);
     }
 
     #[inline]
@@ -1005,6 +974,12 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             self.acc_current_route += 1;
             let current = self.current();
             let cursor = current.cursor_from_ref();
+            let mut dimension = current.dimension;
+
+            // If current has splits then shouldn't use that dimension
+            if self.current_grid().len() > 1 {
+                dimension = self.current_grid().grid_dimension();
+            }
 
             match ContextManager::create_context(
                 (&cursor, current.renderable_content.has_blinking_enabled),
@@ -1012,7 +987,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                 self.window_id,
                 self.acc_current_route,
                 rich_text_id,
-                current.dimension,
+                dimension,
                 &cloned_config,
             ) {
                 Ok(new_context) => {
