@@ -2,14 +2,12 @@ pub mod routes;
 mod window;
 use crate::event::EventProxy;
 use crate::router::window::{configure_window, create_window_builder};
-use crate::scheduler::{Scheduler, TimerId, Topic};
 use crate::screen::{Screen, ScreenWindowProperties};
 use assistant::Assistant;
 use raw_window_handle::{HasDisplayHandle, HasWindowHandle};
 use rio_backend::clipboard::Clipboard;
 use rio_backend::config::Config as RioConfig;
 use rio_backend::error::{RioError, RioErrorLevel, RioErrorType};
-use rio_backend::event::{EventPayload, RioEvent, RioEventType};
 use rio_window::event_loop::ActiveEventLoop;
 use rio_window::keyboard::{Key, NamedKey};
 #[cfg(not(any(target_os = "macos", windows)))]
@@ -58,21 +56,9 @@ impl Route<'_> {
         self.window.winit_window.request_redraw();
     }
 
-    pub fn request_frame(&mut self, scheduler: &mut Scheduler) {
-        let timer_id =
-            TimerId::new(Topic::RenderRoute, self.window.screen.ctx().current_route());
-        let event = EventPayload::new(
-            RioEventType::Rio(RioEvent::Render),
-            self.window.winit_window.id(),
-        );
-
-        if let Some(limit) = self.window.wait_until() {
-            self.window.start_render_timestamp();
-            scheduler.schedule(event, limit, false, timer_id);
-        } else {
-            self.window.start_render_timestamp();
-            self.request_redraw();
-        }
+    #[inline]
+    pub fn begin_render(&mut self) {
+        self.window.render_timestamp = Instant::now();
     }
 
     #[inline]
@@ -393,38 +379,61 @@ impl<'a> RouteWindow<'a> {
         configure_window(&self.winit_window, config);
     }
 
-    pub fn start_render_timestamp(&mut self) {
-        self.render_timestamp = Instant::now();
-    }
-
     pub fn wait_until(&self) -> Option<Duration> {
-        let elapsed_time = Instant::now()
-            .duration_since(self.render_timestamp)
-            .as_millis() as u64;
-        let vblank_interval = self.vblank_interval.as_millis() as u64;
+        let now = Instant::now();
+        let elapsed = now.duration_since(self.render_timestamp);
+        let vblank = self.vblank_interval;
 
-        match vblank_interval >= elapsed_time {
-            true => Some(Duration::from_millis(vblank_interval - elapsed_time)),
-            // false => None,
-            false => Some(Duration::from_millis(vblank_interval.wrapping_sub(1))),
+        // Calculate how many complete frames have elapsed
+        let frames_elapsed = elapsed.as_nanos() / vblank.as_nanos();
+
+        // Calculate when the next frame should occur
+        let next_frame_time = self.render_timestamp
+            + Duration::from_nanos(
+                (frames_elapsed + 1) as u64 * vblank.as_nanos() as u64,
+            );
+
+        if next_frame_time > now {
+            // Return the time to wait until the next ideal frame time
+            Some(next_frame_time.duration_since(now))
+        } else {
+            // We've missed the target frame time, render immediately
+            None
         }
     }
 
+    // TODO: Use it whenever animated cursor is done
+    // pub fn request_animation_frame(&mut self) {
+    //     if self.config.renderer.strategy.is_event_based() {
+    //         // Schedule a render for the next frame time
+    //         let route_id = self.window.screen.ctx().current_route();
+    //         let timer_id = TimerId::new(Topic::RenderRoute, route_id);
+    //         let event = EventPayload::new(
+    //             RioEventType::Rio(RioEvent::RenderRoute(route_id)),
+    //             self.window.winit_window.id(),
+    //         );
+
+    //         // Always schedule at the next vblank interval
+    //         self.scheduler.schedule(event, self.window.vblank_interval, false, timer_id);
+    //     } else {
+    //         // For game loop rendering, the standard redraw is fine
+    //         self.request_redraw();
+    //     }
+    // }
+
     pub fn update_vblank_interval(&mut self) {
         if !self.has_fps_target {
-            // Get the display vblank interval.
-            let monitor_vblank_interval = 1_000_000.
-                / self
-                    .winit_window
-                    .current_monitor()
-                    .and_then(|monitor| monitor.refresh_rate_millihertz())
-                    .unwrap_or(60_000) as f64;
+            // Get the display refresh rate, default to 60Hz if unavailable
+            let refresh_rate_hz = self
+                .winit_window
+                .current_monitor()
+                .and_then(|monitor| monitor.refresh_rate_millihertz())
+                .unwrap_or(60_000) as f64
+                / 1000.0; // Convert millihertz to Hz
 
-            // Now convert it to micro seconds.
-            let monitor_vblank_interval =
-                Duration::from_micros((1000. * monitor_vblank_interval) as u64);
-
-            self.vblank_interval = monitor_vblank_interval;
+            // Calculate frame time in microseconds (1,000,000 µs / refresh_rate)
+            let frame_time_us = (1_000_000.0 / refresh_rate_hz) as u64;
+            self.vblank_interval = Duration::from_micros(frame_time_us);
         }
     }
 
