@@ -1,5 +1,5 @@
 use crate::context::grid::ContextDimension;
-use rio_backend::sugarloaf::{FragmentStyle, Object, Rect, RichText, Sugarloaf};
+use rio_backend::sugarloaf::{FragmentStyle, Object, Quad, RichText, Sugarloaf};
 
 #[inline]
 pub fn screen(sugarloaf: &mut Sugarloaf, context_dimension: &ContextDimension) {
@@ -12,25 +12,32 @@ pub fn screen(sugarloaf: &mut Sugarloaf, context_dimension: &ContextDimension) {
 
     let mut objects = Vec::with_capacity(7);
 
-    objects.push(Object::Rect(Rect {
+    objects.push(Object::Quad(Quad {
         position: [0., 0.0],
         color: black,
-        size: [layout.width, layout.height],
+        size: [
+            layout.width / context_dimension.dimension.scale,
+            layout.height,
+        ],
+        ..Quad::default()
     }));
-    objects.push(Object::Rect(Rect {
+    objects.push(Object::Quad(Quad {
         position: [0., 30.0],
         color: blue,
-        size: [30., layout.height],
+        size: [15., layout.height],
+        ..Quad::default()
     }));
-    objects.push(Object::Rect(Rect {
+    objects.push(Object::Quad(Quad {
         position: [15., context_dimension.margin.top_y + 60.],
         color: yellow,
-        size: [30., layout.height],
+        size: [15., layout.height],
+        ..Quad::default()
     }));
-    objects.push(Object::Rect(Rect {
+    objects.push(Object::Quad(Quad {
         position: [30., context_dimension.margin.top_y + 120.],
         color: red,
-        size: [30., layout.height],
+        size: [15., layout.height],
+        ..Quad::default()
     }));
 
     let heading = sugarloaf.create_temp_rich_text();
@@ -45,14 +52,12 @@ pub fn screen(sugarloaf: &mut Sugarloaf, context_dimension: &ContextDimension) {
     let heading_line = content.sel(heading);
     heading_line
         .clear()
-        .new_line()
         .add_text("Welcome to Rio Terminal", FragmentStyle::default())
         .build();
 
     let paragraph_action_line = content.sel(paragraph_action);
     paragraph_action_line
         .clear()
-        .new_line()
         .add_text(
             "> press enter to continue",
             FragmentStyle {
@@ -71,7 +76,6 @@ pub fn screen(sugarloaf: &mut Sugarloaf, context_dimension: &ContextDimension) {
     let paragraph_line = content.sel(paragraph);
     paragraph_line
         .clear()
-        .new_line()
         .add_text(
             "Your configuration file will be created in",
             FragmentStyle::default(),
@@ -109,16 +113,19 @@ pub fn screen(sugarloaf: &mut Sugarloaf, context_dimension: &ContextDimension) {
     objects.push(Object::RichText(RichText {
         id: heading,
         position: [70., context_dimension.margin.top_y + 30.],
+        lines: None,
     }));
 
     objects.push(Object::RichText(RichText {
         id: paragraph_action,
         position: [70., context_dimension.margin.top_y + 70.],
+        lines: None,
     }));
 
     objects.push(Object::RichText(RichText {
         id: paragraph,
         position: [70., context_dimension.margin.top_y + 140.],
+        lines: None,
     }));
 
     sugarloaf.set_objects(objects);
