@@ -22,7 +22,11 @@ pub struct Atlas {
 }
 
 impl Atlas {
-    pub fn new(device: &wgpu::Device, backend: wgpu::Backend) -> Self {
+    pub fn new(
+        device: &wgpu::Device,
+        backend: wgpu::Backend,
+        context: &crate::context::Context,
+    ) -> Self {
         let layers = match backend {
             // On the GL backend we start with 2 layers, to help wgpu figure
             // out that this texture is `GL_TEXTURE_2D_ARRAY` rather than `GL_TEXTURE_2D`
@@ -37,13 +41,15 @@ impl Atlas {
             depth_or_array_layers: layers.len() as u32,
         };
 
+        let texture_format = context.get_optimal_texture_format(4); // RGBA
+
         let texture = device.create_texture(&wgpu::TextureDescriptor {
             label: Some("image texture atlas"),
             size: extent,
             mip_level_count: 1,
             sample_count: 1,
             dimension: wgpu::TextureDimension::D2,
-            format: wgpu::TextureFormat::Rgba8Unorm,
+            format: texture_format,
             // with gamma correction
             // wgpu::TextureFormat::Rgba8UnormSrgb
             // } else {
@@ -67,6 +73,14 @@ impl Atlas {
         }
     }
 
+    fn get_bytes_per_pixel(&self) -> u32 {
+        match self.texture.format() {
+            wgpu::TextureFormat::Rgba16Float => 8, // F16 RGBA = 8 bytes
+            wgpu::TextureFormat::Rgba8Unorm => 4,  // U8 RGBA = 4 bytes
+            _ => 4,                                // Default fallback
+        }
+    }
+
     pub fn view(&self) -> &wgpu::TextureView {
         &self.texture_view
     }
@@ -75,6 +89,48 @@ impl Atlas {
         self.layers.len()
     }
 
+    pub fn clear(
+        &mut self,
+        device: &wgpu::Device,
+        backend: wgpu::Backend,
+        context: &crate::context::Context,
+    ) {
+        // Reset all layers to empty
+        self.layers = match backend {
+            wgpu::Backend::Gl => vec![Layer::Empty, Layer::Empty],
+            _ => vec![Layer::Empty],
+        };
+
+        // Recreate the texture with the reset layers
+        let extent = wgpu::Extent3d {
+            width: SIZE,
+            height: SIZE,
+            depth_or_array_layers: self.layers.len() as u32,
+        };
+
+        let texture_format = context.get_optimal_texture_format(4); // RGBA
+
+        self.texture = device.create_texture(&wgpu::TextureDescriptor {
+            label: Some("image texture atlas"),
+            size: extent,
+            mip_level_count: 1,
+            sample_count: 1,
+            dimension: wgpu::TextureDimension::D2,
+            format: texture_format,
+            usage: wgpu::TextureUsages::COPY_DST
+                | wgpu::TextureUsages::COPY_SRC
+                | wgpu::TextureUsages::TEXTURE_BINDING,
+            view_formats: &[],
+        });
+
+        self.texture_view = self.texture.create_view(&wgpu::TextureViewDescriptor {
+            dimension: Some(wgpu::TextureViewDimension::D2Array),
+            ..Default::default()
+        });
+
+        tracing::info!("Layer atlas cleared");
+    }
+
     pub fn upload(
         &mut self,
         device: &wgpu::Device,
@@ -82,6 +138,7 @@ impl Atlas {
         width: u32,
         height: u32,
         data: &[u8],
+        context: &crate::context::Context,
     ) -> Option<Entry> {
         let entry = {
             let current_size = self.layers.len();
@@ -96,22 +153,28 @@ impl Atlas {
 
         tracing::info!("Allocated atlas entry: {:?}", entry);
 
+        // Convert input data to the target texture format
+        let converted_data = context.convert_rgba8_to_optimal_format(data);
+        let bytes_per_pixel = self.get_bytes_per_pixel();
+
         // It is a webgpu requirement that:
         //   BufferCopyView.layout.bytes_per_row % wgpu::COPY_BYTES_PER_ROW_ALIGNMENT == 0
         // So we calculate padded_width by rounding width up to the next
         // multiple of wgpu::COPY_BYTES_PER_ROW_ALIGNMENT.
         let align = wgpu::COPY_BYTES_PER_ROW_ALIGNMENT;
-        let padding = (align - (4 * width) % align) % align;
-        let padded_width = (4 * width + padding) as usize;
+        let row_bytes = bytes_per_pixel * width;
+        let padding = (align - row_bytes % align) % align;
+        let padded_width = (row_bytes + padding) as usize;
         let padded_data_size = padded_width * height as usize;
 
         let mut padded_data = vec![0; padded_data_size];
 
         for row in 0..height as usize {
             let offset = row * padded_width;
+            let src_row_bytes = (bytes_per_pixel * width) as usize;
 
-            padded_data[offset..offset + 4 * width as usize].copy_from_slice(
-                &data[row * 4 * width as usize..(row + 1) * 4 * width as usize],
+            padded_data[offset..offset + src_row_bytes].copy_from_slice(
+                &converted_data[row * src_row_bytes..(row + 1) * src_row_bytes],
             )
         }
 
@@ -129,7 +192,7 @@ impl Atlas {
             Entry::Fragmented { fragments, .. } => {
                 for fragment in fragments {
                     let (x, y) = fragment.position;
-                    let offset = (y * padded_width as u32 + 4 * x) as usize;
+                    let offset = (y * padded_width as u32 + bytes_per_pixel * x) as usize;
 
                     self.upload_allocation(
                         &padded_data,
@@ -317,7 +380,9 @@ impl Atlas {
                 buffer: &buffer,
                 layout: wgpu::TexelCopyBufferLayout {
                     offset: offset as u64,
-                    bytes_per_row: Some(4 * image_dimensions.0 + padding),
+                    bytes_per_row: Some(
+                        self.get_bytes_per_pixel() * image_dimensions.0 + padding,
+                    ),
                     rows_per_image: Some(image_dimensions.1),
                 },
             },
