@@ -275,6 +275,12 @@ impl FontLibraryData {
         let mut db = loader::Database::new();
         db.load_system_fonts();
 
+        spec.additional_dirs
+            .unwrap_or_default()
+            .into_iter()
+            .map(PathBuf::from)
+            .for_each(|p| db.load_fonts_dir(p));
+
         match find_font(&db, spec.regular, false, false) {
             FindResult::Found(data) => {
                 self.insert(data);
@@ -443,7 +449,11 @@ impl FontLibraryData {
             self.symbol_maps = Some(symbol_maps);
         }
 
-        fonts_not_fount
+        if spec.disable_warnings_not_found {
+            vec![]
+        } else {
+            fonts_not_fount
+        }
     }
 
     #[cfg(target_arch = "wasm32")]
