@@ -116,7 +116,7 @@ impl Sugarloaf<'_> {
         renderer: SugarloafRenderer,
         font_library: &FontLibrary,
         layout: RootStyle,
-    ) -> Result<Sugarloaf<'a>, SugarloafWithErrors<'a>> {
+    ) -> Result<Sugarloaf<'a>, Box<SugarloafWithErrors<'a>>> {
         let font_features = renderer.font_features.to_owned();
         let ctx = Context::new(window, renderer);
 
@@ -145,6 +145,16 @@ impl Sugarloaf<'_> {
     pub fn update_font(&mut self, font_library: &FontLibrary) {
         tracing::info!("requested a font change");
 
+        // Clear the atlas to remove old font glyphs
+        self.rich_text_brush.clear_atlas();
+
+        // Clear the layer atlas to remove old cached images
+        self.layer_brush.clear_atlas(
+            &self.ctx.device,
+            self.ctx.adapter_info.backend,
+            &self.ctx,
+        );
+
         self.state.reset();
         self.state
             .set_fonts(font_library, &mut self.rich_text_brush);
